Module circular_buffer

Module circular_buffer 

Structs

CircularBuffer
The circular buffer inspired by FASTER’s ring buffer. It acts mostly like a variable length buffer pool, except that evicting entires are handled by the callback.
CircularBufferMetrics
CircularBufferPtr
The guard returned by CircularBuffer::alloc. While this guard is being hold, the allocated memory is not allowed to be evicted. This means that you may block the evicting thread if you hold this guard for too long.
TombstoneHandle
This is a opaque handle that you can use to deallocate the ptr. You need to acquire this handle by CircularBuffer::acquire_exclusive_dealloc_handle before CircularBuffer::dealloc any memory.

Enums

CircularBufferError