pub struct CircularBufferPtr<'a> { /* private fields */ }Expand description
The guard returned by CircularBuffer::alloc. While this guard is being hold, the allocated memory is not allowed to be evicted. This means that you may block the evicting thread if you hold this guard for too long.
The lock will be released once the guard is dropped.
Implementations