pub struct ScanIter<'a, 'b: 'a> { /* private fields */ }Expand description
The scan iterator obtained from [BfTree::scan].
Implementations
Sourceimpl<'b> ScanIter<'_, 'b>
impl<'b> ScanIter<'_, 'b>
pub fn new_with_scan_count( tree: &'b BfTree, start_key: &[u8], scan_cnt: usize, return_field: ScanReturnField, ) -> Self
pub fn new_with_end_key( tree: &'b BfTree, start_key: &[u8], end_key: &[u8], return_field: ScanReturnField, ) -> Self
Sourcepub fn next(&mut self, out_buffer: &mut [u8]) -> Option<(usize, usize)>
pub fn next(&mut self, out_buffer: &mut [u8]) -> Option<(usize, usize)>
Scan next value into out_buffer.
next() terminates if 1) reached the last key. 2) scanned scan_cnt records, if set. 3) reached end_key, if set.
Returns the length of the record fields copied into out_buffer or None if there is no more value.