WalConfig

Struct WalConfig 

pub struct WalConfig { /* private fields */ }

Implementations

Source

impl WalConfig

pub fn new(file_path: impl AsRef<Path>) -> Self

Default: same directory as the file_path

The path to write the write ahead log. Advanced users may want to change the WAL to point to a different location to leverage different storage patterns (WAL is always sequence write and requires durability).

pub fn flush_interval(&mut self, interval: Duration) -> &mut Self

Default: 1ms

pub fn segment_size(&mut self, size: usize) -> &mut Self

Default: 1MB

pub fn storage_backend(&mut self, backend: StorageBackend) -> &mut Self

Default: Std

Change the storage backend for potentially better performance, e.g., IoUring on Linux.