CircularBufferPtr

Struct CircularBufferPtr 

pub struct CircularBufferPtr<'a> { /* private fields */ }
The guard returned by CircularBuffer::alloc. While this guard is being hold, the allocated memory is not allowed to be evicted. This means that you may block the evicting thread if you hold this guard for too long.

The lock will be released once the guard is dropped.

Implementations

Source

impl CircularBufferPtr<'_>

pub fn as_ptr(&self) -> *mut u8

Get the actual pointer to the allocated memory.